/*********************************************************************
* TWI-Master
*
* Uwe Berger; 2008
*
**********************************************************************/
#ifndef F_CPU
	#define F_CPU 12000000
#endif

#include <avr/interrupt.h>
#include "i2clcd.h"

#define LED_DDR  DDRB	
#define LED_PORT PORTB
#define LED_PIN  PB1

#define ADDR_TINY 	0x20

typedef unsigned char uint8_t;

uint8_t weekdays[8][4] = {"xxx\0", "Mo.\0", "Di.\0", "Mi.\0", "Do.\0", "Fr.\0", "Sa.\0", "So.\0"};

volatile uint8_t ss, mm, hh, dd, mt, yy, wd, mez, mesz;
volatile uint8_t hole_werte = 0;
	
//***********************************************************	
ISR(INT1_vect)
{
	// LED-PIN invertieren
	LED_PORT ^= (1 << LED_PIN); 		
	// Merker setzen
	hole_werte = 1;
}	
	
//***********************************************************	
void tiny_read(void)
{
	// Anstoss senden
	i2c_start(ADDR_TINY+I2C_WRITE);
	i2c_write(0);
	i2c_stop();
	// kleine Pause, weil der Tiny-Slave nicht hinterher kommt
	lcd_wait_ms(10);
	// Werte lesen
	i2c_start(ADDR_TINY+I2C_READ);
	ss   = i2c_readAck();
	mm   = i2c_readAck();
	hh   = i2c_readAck();
	dd   = i2c_readAck();
	mt   = i2c_readAck();
	yy   = i2c_readAck();
	wd   = i2c_readAck();
	mez  = i2c_readAck();
	mesz = i2c_readNak();
	i2c_stop();
}

//***********************************************************	
void my_lcd_out (uint8_t py, uint8_t px, uint8_t val, uint8_t fn)
{
	uint8_t x;
	char s[3];

	itoa(val, s, 10);
	x = px;
	if ((val<10)&&(fn)) {
		lcd_printlc(py, x, (unsigned char *)"0");
		x++;
	}
	lcd_printlr(py, x, (unsigned char *)s);	
}

//***********************************************************	
//***********************************************************	
int main(void)
{
	LED_DDR   = (1 << LED_PIN);		// LED-Pin als Ausgang schalten
	LED_PORT &= ~(1 << LED_PIN);		// LED ausschalten
	
	MCUCR |= (1<<ISC10) | (1<<ISC11);	// steigende Flanke (L->H) fuer INT1
	GICR  |= (1<<INT1);          		// externer Interrupt an INT1
	sei(); 					// Interrupts einschalten

	i2c_init();				// TWI initialisieren
	lcd_init();				// LCD initialisieren

	lcd_command(LCD_CLEAR);
	lcd_backlight(ON);
	lcd_printlc(1, 1, (unsigned char *)"DCF77-TWI-Slave ");
	lcd_printlc(2, 1, (unsigned char *)"Uwe Berger; 2008");
	lcd_wait_ms(1000);
	lcd_command(LCD_CLEAR);
	lcd_printlc(1, 3, (unsigned char *)".  .      | |");
	lcd_printlc(2, 3, (unsigned char *)":  :");

	while (true){
		// Interrupt wurde ausgeloest, also Werte via TWI holen/anzeigen
		if (hole_werte) {
			tiny_read();

			my_lcd_out (1, 1, dd, 1);
			my_lcd_out (1, 4, mt, 1);
			my_lcd_out (1, 7, yy, 1);
			lcd_printlc(1, 10, weekdays[wd]);
			//my_lcd_out (1, 10, wd, 1);
			my_lcd_out (1, 14, mez, 0);
			my_lcd_out (1, 16, mesz, 0);
			my_lcd_out (2, 1, hh, 1);
			my_lcd_out (2, 4, mm, 1);
			my_lcd_out (2, 7, ss, 1);

			// Merker zuruecksetzen
			hole_werte = 0;
		}
	}					
}
